@@ -1,6 +1,7 @@
 from .keras_quantization_error import (
     quantization_error as keras_layer_quantization_error,
-    cumulative_quantization_error as keras_cumulative_quantization_error)
+    cumulative_quantization_error as keras_cumulative_quantization_error,
+    weight_quantization_error as keras_weight_quantization_error)
 from .onnx_quantization_error import (
     quantization_error as onnx_node_quantization_error,
     cumulative_quantization_error as onnx_cumulative_quantization_error)
