@@ -1,3 +1,3 @@
-from .generators import *
 from .metrics import *
 from .extractor import *
+from .decorators import *
