@@ -39,7 +39,7 @@ class QuantizedLayer(keras.layers.Layer):
 
     Notes:
         * ``ignored_args`` is a list with the argument names to be skipped
-        * ``arg_constraints`` is a dictionnary where each set of (key, value) corresponds
+        * ``arg_constraints`` is a dictionary where each set of (key, value) corresponds
           to the argument name and the allowed value. For multiple ones, it may be a callable
           that returns the list of values (e.g. lambda: [value1, value2, ...])
     """
@@ -215,7 +215,7 @@ def tensor_inputs(supported):
                 if not inputs.dtype.is_unsigned:
                     in_bits -= 1
                 inputs = FixedPoint(inputs, in_bits, 0)
-            # If the layer is a (Depthwise)BTC the input alignement is handled directly in the
+            # If the layer is a (Depthwise)BTC the input alignment is handled directly in the
             # layer call method, after the input is added to the fifo.
             if self.__class__.__name__ in ['QuantizedBufferTempConv',
                                            'QuantizedDepthwiseBufferTempConv']:
