@@ -17,6 +17,7 @@
 
 __all__ = ["QuantizationParams", "get_quantization_params", "quantization"]
 
+import numpy as np
 from contextlib import contextmanager
 
 
@@ -33,18 +34,28 @@ class QuantizationParams:
         output_bits (int, optional): outputs quantization bitwidth. Defaults to 8.
         input_weight_bits (int, optional): weights quantization bitwidth for the first layer.
             Defaults to 8.
+        input_dtype (np.dtype or str, optional): expected model input format. If given as a string,
+            should follow numpy string type requirements. Defaults to 'uint8'.
         buffer_bits (int, optional): maximal buffer bitwidth allowed in operations.
             Defaults to 32.
+        last_output_bits (int, optional): when provided, sets the bitwidth of the model output. the
+            Defaults to None.
     """
 
     def __init__(self, activation_bits=8, per_tensor_activations=False, weight_bits=8,
-                 output_bits=8, input_weight_bits=8, buffer_bits=32):
+                 output_bits=8, input_weight_bits=8, input_dtype='uint8', buffer_bits=32,
+                 last_output_bits=None):
         self._activation_bits = activation_bits
         self._per_tensor_activations = per_tensor_activations
         self._weight_bits = weight_bits
         self._output_bits = output_bits
         self._input_weight_bits = input_weight_bits
         self._buffer_bits = buffer_bits
+        self._input_dtype = np.dtype(input_dtype)
+        self._last_output_bits = last_output_bits
+        if not np.issubdtype(self._input_dtype, np.integer):
+            raise ValueError(f"Unsupported {self._input_dtype} input dtype: "
+                             "it should be a integer subdtype.")
 
     @property
     def activation_bits(self):
@@ -66,10 +77,18 @@ class QuantizationParams:
     def input_weight_bits(self):
         return self._input_weight_bits
 
+    @property
+    def input_dtype(self):
+        return self._input_dtype
+
     @property
     def buffer_bits(self):
         return self._buffer_bits
 
+    @property
+    def last_output_bits(self):
+        return self._last_output_bits
+
     def __repr__(self) -> str:
         return (
             f"QuantizationParams("
@@ -78,7 +97,9 @@ class QuantizationParams:
             f"weight_bits={self.weight_bits}, "
             f"output_bits={self.output_bits}, "
             f"input_weight_bits={self.input_weight_bits}, "
-            f"buffer_bits={self.buffer_bits})"
+            f"input_dtype={str(self.input_dtype)}, "
+            f"buffer_bits={self.buffer_bits}, "
+            f"last_output_bits={self.last_output_bits})"
         )
 
     def __str__(self) -> str:
@@ -88,7 +109,9 @@ class QuantizationParams:
             f"Weight bits: {self.weight_bits}, "
             f"Output bits: {self.output_bits}, "
             f"Input weight bits: {self.input_weight_bits}, "
-            f"Buffer bits: {self.buffer_bits}"
+            f"Input dtype: {str(self.input_dtype)}, "
+            f"Buffer bits: {self.buffer_bits}, "
+            f"last_output_bits: {self.last_output_bits})"
         )
 
 
