@@ -1,4 +1,4 @@
-from .quantizers import *
+from .base_quantizer import *
 from .output_quantizer import *
 from .weight_quantizer import *
 from .aligned_weight_quantizer import *
