@@ -18,13 +18,14 @@
 __all__ = ["WeightQuantizer"]
 
 import tensorflow as tf
+import tf_keras as keras
 
 from ...tensors import QTensor, QFloat, FixedPoint
 from ..recorders import QFloatRecorder, FixedPointRecorder
-from .quantizers import Quantizer
+from .base_quantizer import Quantizer
 
 
-@tf.keras.utils.register_keras_serializable()
+@keras.saving.register_keras_serializable()
 class WeightQuantizer(Quantizer):
     """A uniform quantizer that converts a float Tensor to a QFloat representation.
 
