@@ -104,7 +104,9 @@ class NonTrackVariable(BaseNonTrackVariable):
             self._var = tf.Variable(init_value, trainable=False,
                                     validate_shape=validate_shape,
                                     name=self._name,
-                                    synchronization=tf.VariableSynchronization.ON_READ)
+                                    dtype=tf.float32,
+                                    synchronization=tf.VariableSynchronization.ON_READ,
+                                    aggregation=tf.VariableAggregation.MEAN)
 
     def reset_var(self):
         """ Reset internal var."""
@@ -157,12 +159,14 @@ class NonTrackFixedPointVariable(BaseNonTrackVariable):
             self._var = tf.Variable(init_value.values, trainable=False,
                                     validate_shape=validate_shape,
                                     name=self._name,
-                                    synchronization=tf.VariableSynchronization.ON_READ)
+                                    synchronization=tf.VariableSynchronization.ON_READ,
+                                    aggregation=tf.VariableAggregation.MEAN)
 
             self._frac_bits = tf.Variable(init_value.frac_bits, trainable=False,
                                           validate_shape=validate_shape,
                                           name=self._name + "/frac_bits",
-                                          synchronization=tf.VariableSynchronization.ON_READ)
+                                          synchronization=tf.VariableSynchronization.ON_READ,
+                                          aggregation=tf.VariableAggregation.MEAN)
 
             self._value_bits = init_value.value_bits
 
@@ -170,7 +174,6 @@ class NonTrackFixedPointVariable(BaseNonTrackVariable):
         """ Reset internal var."""
         if self._var is not None:
             self._var.assign_add(-self._var)
-            self._frac_bits.assign_add(-self._frac_bits)
 
 
 class NonTrackQFloatVariable(BaseNonTrackVariable):
@@ -224,7 +227,6 @@ class NonTrackQFloatVariable(BaseNonTrackVariable):
         """ Reset internal var."""
         if self._scales is not None:
             self._fp.reset_var()
-            self._scales.assign_add(-self._scales)
 
 
 class Recorder():
