@@ -16,7 +16,8 @@
 # ******************************************************************************
 
 __all__ = ["StatefulRecurrent", "QuantizedStatefulRecurrent", "reset_states",
-           "StatefulProjection", "QuantizedStatefulProjection", "update_batch_size"]
+           "StatefulProjection", "QuantizedStatefulProjection", "update_batch_size",
+           "PicoPostProcessing", "QuantizedPicoPostProcessing"]
 
 import tf_keras as keras
 import tensorflow as tf
@@ -110,7 +111,7 @@ class QuantizedStatefulRecurrent(QuantizedLayer, StatefulRecurrent):
         # Build weight quantizer for A_real and A_imag (sharing the same quantizer)
         if "a_quantizer" not in self.quant_config:
             # Forcing to:
-            #   - per-tensor to ensure alignement in the call operations
+            #   - per-tensor to ensure alignment in the call operations
             #   - 16-bits for accuracy reasons
             #   - FixedPoint quantization to prevent scale_out operations on internal_state
             self.quant_config["a_quantizer"] = {"bitwidth": 16, "axis": None, "fp_quantizer": True}
@@ -285,48 +286,21 @@ def update_batch_size(model, batch_size):
 
 @keras.saving.register_keras_serializable()
 class StatefulProjection(keras.layers.Dense):
-    """ Same as a Dense layer but with optional reshaping operation.
-
-    Reshaping can happen both on the inputs and the outputs.
+    """ Same as a Dense layer but with optional subsampling.
 
     Args:
-        downshape (tuple, optional): target shape for downshape operation that happens before the
-            dense. Defaults to None.
-        upshape (tuple, optional): target shape for upshape operation that happens after the dense.
-            Defaults to None.
+        subsample (int, optional): subsampling factor applied to the timesteps. Defaults to 1.
     """
 
-    def __init__(self, *args, downshape=None, upshape=None, subsample=1, **kwargs):
+    def __init__(self, *args, subsample=1, **kwargs):
         super().__init__(*args, **kwargs)
-        self.downshape = tuple(downshape) if downshape else None
-        self.upshape = tuple(upshape) if upshape else None
         self.subsample = subsample
 
     def build(self, input_shape):
         with tf.name_scope(self.name + '/'):
-            if self.downshape is None:
-                super().build(input_shape)
-            else:
-                # When downshape is enabled, build the layer with the target shape so that variables
-                # are build properly
-                super().build(self.downshape)
-                # Edit the input_spec so that from a graph point of view, this layer sees the
-                # original input shape
-                last_dim = tf.TensorShape(input_shape)[-1]
-                self.input_spec = keras.layers.InputSpec(min_ndim=2, axes={-1: last_dim})
-
-    def get_build_config(self):
-        if self._build_input_shape is not None:
-            # If the layer has been built, return the actual input shape
-            # instead of the downshape so that when building from config, the
-            # correct input shape is used
-            return {'input_shape': self.input_shape}
+            super().build(input_shape)
 
     def call(self, inputs):
-        # Apply the optional input downshape
-        if self.downshape is not None:
-            inputs = tf.reshape(inputs, (tf.shape(inputs)[0],) + self.downshape)
-
         # Apply the optional subsampling
         if self.subsample > 1:
             assert inputs.shape[1] % self.subsample == 0, \
@@ -336,16 +310,10 @@ class StatefulProjection(keras.layers.Dense):
 
         # Standard Dense operation
         outputs = super().call(inputs)
-
-        # Apply the optional output upshape
-        if self.upshape is not None:
-            outputs = tf.reshape(outputs, (tf.shape(outputs)[0],) + self.upshape)
         return outputs
 
     def get_config(self):
         config = super().get_config()
-        config["downshape"] = self.downshape
-        config["upshape"] = self.upshape
         config["subsample"] = self.subsample
         return config
 
@@ -362,19 +330,9 @@ class QuantizedStatefulProjection(QuantizedLayer, StatefulProjection):
         self.quant_config['buffer_bitwidth'] = min(28, self.quant_config['buffer_bitwidth'])
         self.buffer_bitwidth = self.quant_config['buffer_bitwidth'] - 1
 
-        # Weight quantizer must be per-tensor to allow upshaping, override it when necessary
-        if self.upshape is not None:
-            self.quant_config["weight_quantizer"]["axis"] = None
-            weight_quantizer_cfg = self.quant_config["weight_quantizer"]
-            self.weight_quantizer = WeightQuantizer(name="weight_quantizer",
-                                                    **weight_quantizer_cfg)
-
     @tensor_inputs([QTensor, tf.Tensor])
     @rescale_outputs
     def call(self, inputs):
-        if self.downshape is not None:
-            inputs = tf.reshape(inputs, (tf.shape(inputs)[0],) + self.downshape)
-
         if self.subsample > 1:
             assert inputs.shape[1] % self.subsample == 0, \
                 f"Number of timesteps: {inputs.shape[1]} must be a multiple of subsample " \
@@ -391,6 +349,114 @@ class QuantizedStatefulProjection(QuantizedLayer, StatefulProjection):
             bias = self.bias_quantizer(self.bias, outputs)
             outputs = tf.add(outputs, bias)
 
-        if self.upshape is not None:
-            outputs = tf.reshape(outputs, (tf.shape(outputs)[0],) + self.upshape)
         return outputs
+
+
+@keras.saving.register_keras_serializable()
+class PicoPostProcessing(keras.layers.Layer):
+    """ Post-processing layer that computes the mean absolute difference between predictions and
+    targets along a given axis and binarizes the result using a threshold.
+
+    This layer is useful for light-weight evaluation where a continuous error metric is
+    converted into a binary decision per sample (e.g. anomaly detection or simple pass/fail).
+
+    Args:
+        threshold (float): threshold applied to the mean absolute difference. Values greater
+            than or equal to it become 1.0, otherwise 0.0.
+
+    Example:
+        >>> layer = PicoPostProcessing(threshold=0.5)
+        >>> out = layer(y_pred, y_true)
+    """
+
+    def __init__(self, *args, threshold, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.threshold = threshold
+
+    def _check_input_constraints(self, y_pred, y_true):
+        # Check input constraints.
+        if not isinstance(y_true, tf.Tensor):
+            raise TypeError(f"{self.__class__.__name__} only accepts {tf.Tensor} for second input. "
+                            f"Receives '{type(y_true)}'.")
+
+        if not (y_true_dtype := tf.as_dtype(y_true.dtype)).is_integer:
+            raise TypeError(f"{self.__class__.__name__} only accepts integer for second input. "
+                            f"Receives '{y_true_dtype}'.")
+
+    def call(self, y_pred, y_true):
+        self._check_input_constraints(y_pred, y_true)
+
+        # Compare inputs.
+        y_true = tf.cast(y_true, y_pred.dtype)
+        outputs = tf.reduce_mean(tf.abs(y_pred - y_true), axis=1)
+
+        # Binarize the output.
+        outputs = tf.where(outputs >= self.threshold, 1.0, 0.0)
+        return outputs
+
+    def get_config(self):
+        config = super().get_config()
+        config["threshold"] = self.threshold
+        return config
+
+
+@register_quantize_target([PicoPostProcessing])
+@keras.saving.register_keras_serializable()
+class QuantizedPicoPostProcessing(QuantizedLayer, PicoPostProcessing):
+    """Quantized PicoPostProcessing layer.
+
+    A quantized variant of PicoPostProcessing that operates on quantized predictions
+    (QTensor) and integer targets.
+
+    Args:
+        threshold (float): threshold applied to the mean absolute difference. Values greater
+            than or equal to it become 1.0, otherwise 0.0.
+    """
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+
+        # Promote buffer bitwidth to 44.
+        self.quant_config['buffer_bitwidth'] = max(44, self.quant_config.get('buffer_bitwidth', 0))
+        self.buffer_bitwidth = self.quant_config['buffer_bitwidth'] - 1
+
+        # Initialize prediction quantizer with default parameters.
+        self.pred_quantizer = OutputQuantizer(bitwidth=16,
+                                              axis="per-axis",
+                                              scale_bits=16,
+                                              buffer_bitwidth=self.quant_config['buffer_bitwidth'],
+                                              name="prediction_quantizer")
+
+    def build(self, input_shape):
+        super().build(input_shape)
+        with tf.name_scope(self.name + '/'):
+            # Explicitly build the OutputQuantizers so that output frac_bits can be computed.
+            self.pred_quantizer.build(input_shape)
+
+    def _check_input_constraints(self, y_pred, y_true):
+        super()._check_input_constraints(y_pred, y_true)
+        if not isinstance(y_pred, QTensor):
+            raise TypeError(f"{self.__class__.__name__} only accepts {QTensor} for first input. "
+                            f"Receives '{type(y_pred)}'.")
+
+    def call(self, y_pred, y_true):
+        self._check_input_constraints(y_pred, y_true)
+
+        # Force quantizer to rescale the y_pred to '0' frac_bits, as this is the input scale.
+        # This is achieved by setting the maximum range to the limit.
+        max_range = 2.0**self.pred_quantizer.value_bits
+        self.pred_quantizer.range_max.assign(
+            max_range * tf.ones_like(self.pred_quantizer.range_max))
+
+        # Convert both inputs into a FixedPoint.
+        y_true = FixedPoint(y_true, value_bits=self.pred_quantizer.value_bits, frac_bits=0)
+        y_pred = self.pred_quantizer(y_pred)
+
+        # Compare inputs.
+        outputs = tf.abs(y_pred - y_true)
+        outputs = outputs.promote(self.buffer_bitwidth)
+        outputs = tf.reduce_mean(outputs, axis=1)
+
+        # Binarize the output.
+        outputs = tf.where(outputs.values >= self.threshold, 1.0, 0.0)
+        return FixedPoint(outputs, value_bits=1, frac_bits=0)
