@@ -31,7 +31,7 @@ from .transforms.align_rescaling import _fold_rescaling
 from .utils import apply_weights_to_model
 from ..layers import (InputObserver, OutputObserver, PaddedConv2D, DepthwiseConv2DTranspose,
                       BufferTempConv, DepthwiseBufferTempConv, StatefulRecurrent,
-                      StatefulProjection, InputQuantizer, Dequantizer, update_batch_size)
+                      InputQuantizer, Dequantizer, update_batch_size)
 from ..tensors import FixedPoint, floor_log2
 
 
@@ -100,9 +100,6 @@ def _get_next_layer(layer, supported_layers, skippable_layers=(), reshape_ops=[]
         return None
     next_layer = outbounds[0].layer
 
-    # StatefulProjection with downshape or upshape is not supported by CLE
-    if isinstance(next_layer, StatefulProjection) and (next_layer.downshape or next_layer.upshape):
-        return None
     # If the layer is supported, it is a valid candidate
     if isinstance(next_layer, supported_layers):
         return next_layer
@@ -155,7 +152,7 @@ def _set_and_equalize(qmodel, layer_name, range_max):
     # happen
     reshape_ops = []
     next_layer = _get_next_layer(target_layer, supported_layers, skippable_layers, reshape_ops)
-    # For now cross layer equalization for (Dephwise)BufferTempConvLayers is possible only if their
+    # For now cross layer equalization for (Depthwise)BufferTempConvLayers is possible only if their
     # input is quantized per-tensor.
     # cross layer equalization is not applied on conv2d when groups is greater than 1 and when
     # data_format is 'channels_first'
@@ -318,12 +315,12 @@ def calibrate(model, qmodel, samples=None, num_samples=1024, batch_size=None, ep
                             " data (eg number of frames per video clips). \n"
                             " Recurrent expects [batch_size, (model.input_shape)] array.")
         if buf_layers:
-            # With ST TENN models, the number of samples is infered from the provided samples
+            # With ST TENN models, the number of samples is inferred from the provided samples
             num_samples = len(samples)
             if batch_size is None:
                 raise ValueError("batch_size not specified. With spatiotemporal TENN models "
                                  "batch_size must be given for the calibration, otherwise the "
-                                 "result is unpredictible. Refer to 'calibrate' function docstring "
+                                 "result is unpredictable. Refer to 'calibrate' function docstring "
                                  "for more details.")
 
             # The number of samples must be a multiple of the batch_size to prevent a reset of the
