@@ -23,11 +23,13 @@ __all__ = ["align_rescaling"]
 import numpy as np
 
 from copy import deepcopy
-from keras.layers import Rescaling, Conv2D, Dense
+from tf_keras.layers import Rescaling, Conv2D, Dense
+from tf_keras.saving import serialize_keras_object
 
-from .transforms_utils import get_layers_by_type, get_layers
-from ..utils import apply_weights_to_model
-from ...layers.convolution import PaddedConv2D
+from .transforms_utils import get_layers_by_type, get_layers, safe_fail
+from ..utils import apply_weights_to_model, requires_tf_keras_model
+from ...layers.convolution import PaddedConv2D, QuantizedConv2D
+from ...layers import BufferTempConv, DepthwiseBufferTempConv
 
 
 def _find_rescaling_fold_target(rescaling):
@@ -41,7 +43,7 @@ def _find_rescaling_fold_target(rescaling):
         otherwise.
     """
     # Define layers that can accept Rescaling alignment
-    supported_dst_layers = [Conv2D, Dense]
+    supported_dst_layers = [Conv2D, Dense, BufferTempConv, DepthwiseBufferTempConv]
 
     scale_per_axis = isinstance(rescaling.scale, (list, tuple)) and len(rescaling.scale) > 1
     if isinstance(rescaling.offset, (list, tuple)):
@@ -63,12 +65,11 @@ def _find_rescaling_fold_target(rescaling):
     return dst_layer
 
 
-def _adapt_padding(model, offset, dst_layer):
-    """ Update padding scheme by replacing Conv2D with a PaddedConv2D and appropriate padding value.
+def _adapt_padding(model, dst_layer):
+    """ Update padding scheme by replacing Conv2D with a PaddedConv2D.
 
     Args:
         model (keras.Model): the original model
-        offset (float, list, tuple): the offset to fold
         dst_layer (keras.layer): the layer where offset or scale will be folded
 
     Returns:
@@ -79,29 +80,27 @@ def _adapt_padding(model, offset, dst_layer):
     # Fold rescaling by editing the model configuration
     dst_config = get_layers(config, [dst_layer.name])[0]
     # Force bias
+    had_bias = dst_config['config']['use_bias']
     dst_config['config']['use_bias'] = True
-    # Replace Conv2D with 'same' padding by PaddedConv2D with correct padding value
+    # Replace Conv2D with 'same' padding by PaddedConv2D
     if isinstance(dst_layer, Conv2D) and dst_layer.padding.lower() == 'same':
-        if isinstance(offset, (list, tuple)):
-            chan = dst_layer.get_weights()[0].shape[2]
-            assert len(offset) in (1, chan), "offset must be a scalar or of size of channels"
-            pad_values = [float(-p) for p in offset]
-        else:
-            pad_values = float(-offset)
-
-        dst_config['config']['padding_value'] = pad_values
-        dst_config['class_name'] = 'Custom>PaddedConv2D'
+        new_config = PaddedConv2D.from_config(dst_config['config'])
+        dst_config.update(serialize_keras_object(new_config))
+        dst_config['config']['padding_value'] = [0.0]
 
     # Reconstruct model from the config
     aligned_model = model.from_config(config)
 
-    # Restore model weights
+    # Restore model weights, setting bias to zero if they was created.
     variables_dict = {var.name: var for var in model.variables}
     apply_weights_to_model(aligned_model, variables_dict, False)
+    if not had_bias:
+        new_bias = aligned_model.get_layer(dst_layer.name).bias
+        new_bias.assign_sub(new_bias)
     return aligned_model
 
 
-def _fold_rescaling(rescaling_layer, dst_layer, had_bias):
+def _fold_rescaling(rescaling_layer, dst_layer):
     """ Fold rescaling parameters into dst_layer weights and bias.
 
     Note that if scales are per-channel, there are folded in the weights.
@@ -109,38 +108,52 @@ def _fold_rescaling(rescaling_layer, dst_layer, had_bias):
     Args:
         rescaling_layer (keras.layers.Layer): the Rescaling layer
         dst_layer (keras.layers.Layer): the layer where Rescaling is folded
-        had_bias (bool): whether the original layer had a bias or not
     """
-    assert isinstance(dst_layer, (Conv2D, Dense))
-    base_weights = dst_layer.get_weights()
-    new_w = base_weights[0].copy()
-    filters = new_w.shape[-1]
+    assert isinstance(dst_layer, (Conv2D, Dense, BufferTempConv, DepthwiseBufferTempConv))
+    kernel = dst_layer.kernel
+    assert (bias := dst_layer.bias) is not None, f"{dst_layer.name} must have a bias."
 
+    # Fold offset in padding_value.
+    offset = np.array(rescaling_layer.offset)
+    if (isinstance(dst_layer, (PaddedConv2D, QuantizedConv2D)) and
+            dst_layer.padding_value is not None):
+        # Update previous padding value.
+        dst_layer._padding_value = (np.array(dst_layer._padding_value) - offset).tolist()
+        # Reduce to just one value if possible (improve inference runtime).
+        if all(x == dst_layer._padding_value[0] for x in dst_layer._padding_value):
+            dst_layer._padding_value = [dst_layer._padding_value[0]]
+
+    # Fold offset in bias.
+    vector_offset = (isinstance(rescaling_layer.offset, (list, tuple)) and
+                     len(rescaling_layer.offset) > 1)
+    if vector_offset and isinstance(dst_layer, BufferTempConv):
+        # Reshape the offset
+        offset = np.tile(offset, dst_layer.kernel_size)
+    if not isinstance(dst_layer, DepthwiseBufferTempConv):
+        offset = np.reshape(offset, newshape=(-1, 1))
+    bias.assign_add(np.sum(kernel * offset, axis=tuple(range(np.ndim(kernel) - 1))))
+    rescaling_layer.offset = 0
+
+    # Fold scale in kernel when it is a vector.
     scale = rescaling_layer.scale
-    if isinstance(scale, (list, tuple)) and len(scale) > 1:
-        # If scale is not a scalar, align it
+    vector_scale = isinstance(scale, (list, tuple)) and len(scale) > 1
+    if vector_scale:
         rescaling_layer.scale = 1
-        for i in range(filters):
-            # To compensate, adjust the weights of the next layer
-            new_w[..., i] *= scale
+        scale = np.array(scale)
         if isinstance(dst_layer, PaddedConv2D):
             # Also rescale the padding value
             padding_value = np.array(dst_layer._padding_value, dtype=np.float32)
-            dst_layer._padding_value = padding_value / scale
-    new_weights = [new_w]
+            dst_layer._padding_value = list(padding_value / scale)
+        elif isinstance(dst_layer, (BufferTempConv)):
+            scale = np.tile(scale, dst_layer.kernel_size)
+        if not isinstance(dst_layer, DepthwiseBufferTempConv):
+            scale = np.reshape(scale, newshape=(-1, 1))
 
-    if dst_layer.use_bias:
-        # Build zero initialized biases if the original layer didn't have any
-        new_biases = base_weights[1].copy() if had_bias else np.zeros(filters)
-        for i in range(filters):
-            # Rescale biases filter by filter to enable broadcast if offsets are per channel
-            w_i = base_weights[0][..., i]
-            new_biases[i] += np.sum(w_i * rescaling_layer.offset)
-        new_weights += [new_biases]
-        rescaling_layer.offset = 0
-    dst_layer.set_weights(new_weights)
+        kernel.assign(kernel * scale)
 
 
+@safe_fail
+@requires_tf_keras_model
 def align_rescaling(model):
     """Aligns the Rescaling layer of the model to make it quantization ready.
 
@@ -174,11 +187,9 @@ def align_rescaling(model):
         return model
 
     # There is a rescaling offset, dst_layer padding scheme must be updated
-    offset = rescaling_layer.offset
-    aligned_model = _adapt_padding(model, offset, dst_layer)
+    aligned_model = _adapt_padding(model, dst_layer)
 
     # Fold Rescaling parameters into the new layer weights
     _fold_rescaling(aligned_model.get_layer(rescaling_layer.name),
-                    aligned_model.get_layer(dst_layer.name),
-                    dst_layer.use_bias)
+                    aligned_model.get_layer(dst_layer.name))
     return aligned_model
