@@ -21,9 +21,27 @@ Helper that replaces lambdas with their equivalent Keras layer.
 __all__ = ["replace_lambda"]
 
 from copy import deepcopy
-from keras.layers import Activation, TFOpLambda
+from tf_keras import layers
+from tf_keras.src.layers import TFOpLambda
 
-from .transforms_utils import get_layers_by_type, get_layer_index, inbound_node_generator
+from .transforms_utils import (get_layers_by_type, get_layer_index, inbound_node_generator,
+                               update_layer_serialization, safe_fail)
+from ..utils import requires_tf_keras_model
+from ...layers import Activation
+
+
+def _is_layer_config(inbound):
+    return (isinstance(inbound, list)
+            and len(inbound) == 3
+            and all(isinstance(x, t) for x, t in zip(inbound, (str, int, int))))
+
+
+def _is_valid_lambda_layer(layer_config):
+    if layer_config['config']['function'] not in ['__operators__.add', '__operators__.multiply',
+                                                  'math.add', 'math.multiply']:
+        return True
+    inbounds = layer_config['inbound_nodes'][0]
+    return inbounds[0] != '_CONSTANT_VALUE' and _is_layer_config(inbounds[-1]['y'])
 
 
 def _update_inbound_nodes(layer_config, target):
@@ -48,15 +66,18 @@ def _update_inbound_nodes(layer_config, target):
         # "connection_info[-1]" holds the lambda config, that is the op name and other lambda
         # specific parameters. Code below will retrieve meaningful parameters that will be used to
         # define the config of the new layer and will then be dropped.
-        if target == 'Reshape':
+        if target == layers.Reshape:
             # Set the 'target_shape' parameter from the 'shape' attribute
             layer_config['config']['target_shape'] = connection_info[-1].get('shape')
-        elif target == 'Permute':
+        elif target == layers.Permute:
             # Set the 'dims' parameter from the 'perm' attribute
             perm = connection_info[-1].get('perm')
             # Permute 'dims' must start at 1 but transpose 'perm' starts at 0
             layer_config['config']['dims'] = [p + 1 for p in perm]
-        elif target == "Add":
+        elif target == Activation:
+            # Include lambda extra parameters in config
+            layer_config['config'].update(connection_info[-1])
+        elif target in [layers.Add, layers.Multiply]:
             # Get the second inbound currently defined as {'y': ['name', 0, 0]}
             other_inbound = connection_info[-1].get('y')
             # Modify it to fit the ['name', 0, 0, {}] convention
@@ -64,6 +85,7 @@ def _update_inbound_nodes(layer_config, target):
             # Add the updated inbound into the inbound_node list and set it in layer_config
             inbound_node.append(other_inbound)
             layer_config['inbound_nodes'] = [inbound_node]
+
         # Retrieve the lambda name as it will be used to set the name for the layer
         name = connection_info[-1].get('name', None)
         # Drop the lambda config
@@ -71,6 +93,8 @@ def _update_inbound_nodes(layer_config, target):
     return name
 
 
+@safe_fail
+@requires_tf_keras_model
 def replace_lambda(model):
     """ Replaces lambda layers from a model with their equivalent Keras layer.
 
@@ -80,8 +104,9 @@ def replace_lambda(model):
         - Lambda(transpose) → Permute,
         - Lambda(reshape) → Reshape,
         - Lambda(add) → Add,
-        - Lambda('gelu') → Activation('gelu'),
-        - Lambda('silu') → Activation('silu').
+        - Lambda(gelu) or Activation(gelu) → quantizeml.layers.Activation(gelu),
+        - Lambda(silu) or Activation(silu) → quantizeml.layers.Activation(silu),
+        - Lambda(leaky_relu) or Activation(leaky_relu) → quantizeml.layers.Activation(leaky_relu).
 
     Args:
         model (keras.Model): the model of interest
@@ -91,18 +116,22 @@ def replace_lambda(model):
     """
     # Map function names to Keras layers
     lambda_to_layer = {
-        'nn.relu': 'ReLU',
-        'math.add': 'Add',
-        '__operators__.add': 'Add',
-        'reshape': 'Reshape',
-        'transpose': 'Permute',
-        'compat.v1.transpose': 'Permute',
-        'nn.silu': 'Activation',
-        'nn.gelu': 'Activation',
+        'nn.relu': layers.ReLU,
+        'nn.relu6': layers.ReLU(max_value=6),
+        'math.add': layers.Add,
+        '__operators__.add': layers.Add,
+        'math.multiply': layers.Multiply,
+        '__operators__.multiply': layers.Multiply,
+        'reshape': layers.Reshape,
+        'transpose': layers.Permute,
+        'compat.v1.transpose': layers.Permute,
+        'nn.silu': Activation,
+        'nn.gelu': Activation,
+        'nn.leaky_relu': Activation,
     }
 
     # Get all Activations and TFOpLambda layers present in the model
-    lambdas = get_layers_by_type(model, (Activation, TFOpLambda))
+    lambdas = get_layers_by_type(model, (layers.Activation, TFOpLambda))
 
     # When there are no valid candidates, return the original model
     if not lambdas:
@@ -110,6 +139,7 @@ def replace_lambda(model):
 
     # Copy configuration before applying modifications
     config = deepcopy(model.get_config())
+    supported_acts = Activation.arg_constraints['activation']()
 
     for layer in lambdas:
         layer_index = get_layer_index(config['layers'], layer.name)
@@ -119,22 +149,27 @@ def replace_lambda(model):
                 and layer_config['config']['activation'] == 'relu'):
             # Drop the 'activation' parameter and update 'class_name'
             layer_config['config'].pop('activation')
-            layer_config['class_name'] = 'ReLU'
-        # Replace TFOpLambda layers
-        elif layer_config['class_name'] == 'TFOpLambda':
+            update_layer_serialization(layer_config, layers.ReLU)
+        # Replace Activation layers by quantizeml.layers.Activation
+        elif (layer_config['class_name'] == 'Activation'
+              and layer_config['config']['activation'] in supported_acts):
+            update_layer_serialization(layer_config, Activation)
+        # Replace TFOpLambda layers except add and multiply if one of their inbounds is not a layer
+        elif layer_config['class_name'] == 'TFOpLambda' and _is_valid_lambda_layer(layer_config):
             # Retrieve the function used in the config and get the equivalent Keras layer name
             target = lambda_to_layer.get(layer_config['config']['function'], None)
             if target:
                 # Drop the 'function' parameter and update 'class_name'
                 op_name = layer_config['config'].pop('function').replace('nn.', '')
-                layer_config['class_name'] = target
                 # If target is 'Activation', put op_name in 'activation' parameter
-                if target == "Activation":
+                if target == Activation:
                     layer_config['config']['activation'] = op_name
                 # Update the inbound part of the config: the last element of the inbound list of
                 # lambda layers will contain the lambda op parameters that are used to set the
                 # config for the new layer.
                 new_name = _update_inbound_nodes(layer_config, target)
+                # Update serialization
+                update_layer_serialization(layer_config, target)
                 # If layer name was updated, use the new name everywhere in the config
                 if new_name:
                     # Serialize the dict into a string
@@ -144,6 +179,8 @@ def replace_lambda(model):
                     # Deserialize the updated string into a dict
                     config = eval(str_config)
 
+    if config == model.get_config():
+        return model
     # Reconstruct model from the config
     updated_model = model.from_config(config)
 
