@@ -77,7 +77,7 @@ def sanitize_base(model):
 
 @requires_tf_keras_model
 def sanitize_for_hardware(model):
-    """ Perform sanitizing steps targetting hardware compatiblity.
+    """ Perform sanitizing steps targeting hardware compatibility.
 
     Akida hardware comes with some limitations that can be avoided by smart transformations on the
     model.
