@@ -17,6 +17,7 @@
 """
 Transforms utility methods.
 """
+import functools
 from copy import deepcopy
 
 from tf_keras.models import Sequential
@@ -123,7 +124,7 @@ def update_inbound(layer_config, name, updated_inbound):
     """ Update the layer 'name' inbound in config with the provided name.
 
     Args:
-        layer_config (dict): layer config to udpate
+        layer_config (dict): layer config to update
         name (str): name of the inbound to replace
         updated_inbound (str): new inbound name
     """
@@ -275,6 +276,7 @@ def safe_fail(func):
     Returns:
         callable: the decorated function.
     """
+    @functools.wraps(func)
     def decorator(model):
         try:
             return func(model)
