@@ -20,6 +20,7 @@ Common utility methods used in quantization models.
 
 __all__ = ['apply_weights_to_model', 'requires_tf_keras_model']
 
+import functools
 import warnings
 import tf_keras as keras
 
@@ -34,6 +35,7 @@ def requires_tf_keras_model(func):
     Returns:
         Callable: the decorated function.
     """
+    @functools.wraps(func)
     def wrapper(model, *args, **kwargs):
         if not isinstance(model, keras.Model):
             raise ValueError(
