@@ -34,7 +34,7 @@ BRN_OPSET = Opset(domain="com.brainchip", version=1)
 # Normally onnxruntime supports up to opset 22, but some optimizations are not
 # applied (e.g. Fold batchnorms), so we limit it to 21 for now.
 ONNX_OPSET = Opset(domain="", version=21)
-IR_VERSION = 10
+IR_VERSION = 11
 
 # List of quantized layers that requires a power of two scale
 _GLOBAL_REQUIRES_FP_INPUTS = []
@@ -81,7 +81,7 @@ class OnnxLayer:
         base_name (str): the operation type base name.
         name (str, optional): the node name. Defaults to ''.
         kwargs (dict, optional): the custom attributes. Each attribute type will be
-            infered by ``onnx.helper.make_attribute()``. Defaults to {}.
+            inferred by ``onnx.helper.make_attribute()``. Defaults to {}.
     """
 
     def __init__(self, base_name, name='', **kwargs):
@@ -172,7 +172,7 @@ class OnnxLayer:
             value = np.array(value)
         if name not in self.weights:
             raise ValueError(f"{self.name} ({self.base_name}) does not recognize '{name}'. "
-                             f"Availables: {list(self.weights)}")
+                             f"Available: {list(self.weights)}")
         if value.dtype != self.weights[name].dtype:
             raise ValueError(f"{self.base_name}/{name} does not match with expected type "
                              f"({self._weights[name].dtype}). Receives {value.dtype}")
