@@ -28,13 +28,28 @@ from .subgraph_ops.padding import get_padding_ops, transform_pads_into_array
 from .compute_shapes import compute_onnx_conv_output
 from .set_weights import set_weights_on_qnode, set_max_value_on_qnode, set_range_max_on_qnode
 from ..graph_tools import (TENSOR_SHAPE, get_field, get_node, get_activation,
-                           get_tensor_shape, check_node_attributes)
+                           get_tensor_shape, check_node_attributes, get_variable)
 from ..quantization.core import (quantize_to_qfloat, aligned_quantize, fold_zero_point, align_to,
                                  downscale, compute_lut_values, dequantize)
 
 
 def get_qconv(nodes, graph, tensor_ranges):
+    # Resolve Conv and optional Pad
     conv_node = nodes[0]
+    pad_node = get_node(nodes, "BRNPad")
+
+    pads = None
+    padding_value = None
+
+    if pad_node:
+        conv_node = nodes[1]
+        padding_value = get_variable(pad_node.input[2], graph)
+        pads = get_variable(pad_node.input[1], graph)
+    else:
+        pads_attr = get_field(conv_node, "pads", False)
+        if pads_attr:
+            pads = transform_pads_into_array(pads_attr)
+
     assert conv_node.op_type == 'Conv'
 
     # Check supported attributes
@@ -72,9 +87,12 @@ def get_qconv(nodes, graph, tensor_ranges):
 
     # Sets the weights to configure the operation chain
     set_weights_on_qnode(qconv, conv_node, graph)
-    pads = get_field(conv_node, 'pads', False)
-    if pads:
-        qconv.set_weight("pads", transform_pads_into_array(pads))
+    if pad_node:
+        qconv.set_weight("padding_value", padding_value)
+
+    if pads is not None:
+        qconv.set_weight("pads", pads)
+
     if act_node.op_type == "Clip":
         set_max_value_on_qnode(qconv, act_node, graph)
 
@@ -136,6 +154,7 @@ class QuantizedConv2D(OnnxLayer):
         self._add_weight("bias")
         self._add_weight("max_value")
         self._add_weight("pads", dtype="int64")
+        self._add_weight("padding_value")
         self._add_weight("range_max", 1.0)
         self._add_weight("act_range_max", 1.0)
 
@@ -157,6 +176,9 @@ class QuantizedConv2D(OnnxLayer):
         if self.weights["pads"].size == 0:
             self.set_weight("pads", np.zeros(len(kernel_shape) * 2, dtype="int64"))
 
+        if self.weights["padding_value"].size == 0:
+            self.set_weight("padding_value", np.zeros(kernel_shape[1], dtype="float32"))
+
         # Compute output shape
         conv_output_shape = compute_onnx_conv_output(self, input_ts.shape)
         output_ts = TENSOR_SHAPE(conv_output_shape, np.dtype("int8"))
@@ -180,11 +202,15 @@ class QuantizedConv2D(OnnxLayer):
         weights_dict = {}
         bias = self.weights["bias"]
         if "InputConv" in self.op_type:
-            # If calibration was done per tensor, repeat zero point over each channel
             zero_point = qinput.weights["zero_point"]
             if zero_point.size == 1:
                 zero_point = np.repeat(zero_point, kernel.shape[1])
-            weights_dict[prefix + "Xpad"] = zero_point
+
+            # Quantize padding value
+            qpad_value = aligned_quantize(self.weights["padding_value"],
+                                          qinput.weights['scale'], signed=False)
+            weights_dict[prefix + "Xpad"] = zero_point + qpad_value
+
             # Fold zero point in bias
             # Note: Dequantize kernel instead to use the float to reduce quantization error
             kernel = dequantize(qweights, i_scale, axis=0)
@@ -226,7 +252,7 @@ class QuantizedConv2D(OnnxLayer):
         weights_dict.update({prefix + "M": align_to(scale.astype("uint8"), qweights.ndim),
                              prefix + "S_out": align_to(s_out, qweights.ndim)})
 
-        # Return quantized weights and ouput scale
+        # Return quantized weights and output scale
         return weights_dict, ocalib_scale
 
     @staticmethod
@@ -254,7 +280,7 @@ class QuantizedConv2D(OnnxLayer):
         # LUT (optional)
         if "LUT" in op_type:
             nodes[-1].output.__setitem__(0, nodes[-1].op_type)
-            # Intermedial downscale
+            # Intermediate downscale
             nodes += get_scale_out_ops(nodes[-1].output[0], "Ys", scale_name="ActScale",
                                        shift_name="ActShift", bitwidth=11)
             # Main operation
