@@ -25,15 +25,16 @@ from onnx.helper import make_node, make_tensor_value_info, np_dtype_to_tensor_dt
 from onnxruntime.quantization.calibrate import TensorData
 
 from ...models import get_quantization_params
-from ..graph_tools import (TENSOR_SHAPE, array_to_tp, get_field, get_node, get_variable, has_field,
+from ..graph_tools import (TENSOR_SHAPE, array_to_tp, get_field, get_node, has_field,
                            to_field, value_info_to_tensor_shape)
 from ..quantization.core import input_zp_scale, round_to_nearest_pow2
 from .base_layer import OnnxLayer
-from .layer_compatibility import check_node_has_one_initializer, check_node_link_to_input
+from .layer_compatibility import check_node_link_to_input
 
 
 def get_input_quantizer(nodes, graph, tensor_ranges):
-    check_node_link_to_input(nodes[0], graph)
+    if nodes:
+        check_node_link_to_input(nodes[0], graph)
 
     if (cast_node := get_node(nodes, "Cast")) is not None:
         assert get_field(cast_node, "to") == TensorProto.FLOAT, "Cast node dtype must be float32"
@@ -50,48 +51,16 @@ def get_input_quantizer(nodes, graph, tensor_ranges):
                                      input_signed=input_signed,
                                      perm=perm)
 
-    # Sets rescaling weights
-    if (mul_node := get_node(nodes, "Mul")) is not None:
-        check_node_has_one_initializer(mul_node, graph)
-        input_quantizer.set_weight("input_scale", get_variable(mul_node.input[1], graph))
-    if (add_node := get_node(nodes, "Add")) is not None:
-        check_node_has_one_initializer(add_node, graph)
-        input_quantizer.set_weight("offset", get_variable(add_node.input[1], graph))
-
     # Set calibration ranges
-    out_ranges = tensor_ranges[nodes[-1].output[0]]
-    input_quantizer.set_weight("range_min", out_ranges.lowest)
-    input_quantizer.set_weight("range_max", out_ranges.highest)
+    # The range key is the output of the last node ("Cast" or "Transpose") if nodes is not empty,
+    # otherwise it is the name of the graph input.
+    range_key = nodes[-1].output[0] if nodes else graph.input[0].name
+    ranges = tensor_ranges[range_key]
+    input_quantizer.set_weight("range_min", ranges.lowest)
+    input_quantizer.set_weight("range_max", ranges.highest)
     return input_quantizer
 
 
-def _compute_quantization_parameters(r_scale, r_offset, out_tensor_range, signed):
-    assert isinstance(out_tensor_range, TensorData)
-    # Computing the ranges before the Rescale node (e.g. the model inputs)
-
-    in_tensor_range = [(x - r_offset) / r_scale for x in out_tensor_range.range_value]
-    in_tensor_range = TensorData(lowest=np.minimum(*in_tensor_range),
-                                 highest=np.maximum(*in_tensor_range))
-
-    # Compute scale and zero point to quantize the inputs
-    input_scale, input_zp = input_zp_scale(in_tensor_range, allow_zp=not signed)
-
-    # Compute scale and output to dequantize the outputs
-    output_scale, output_zp = input_zp_scale(out_tensor_range, allow_zp=not signed)
-
-    # Check constraints
-    err_msg = "Impossible to quantize inputs when folding rescale: "
-    if not signed:
-        # Compare if rescale is valid
-        np.testing.assert_allclose(input_scale / output_scale,
-                                   r_scale,
-                                   atol=1e-3,
-                                   err_msg=err_msg + "input/output scales ratio is not valid.")
-    else:
-        np.testing.assert_equal(r_offset, 0, err_msg + "offset must be zero when input is signed.")
-    return (input_scale, input_zp), (output_scale, output_zp)
-
-
 class InputQuantizer(OnnxLayer):
     """Intermediate representation of QuantizeLinear(), use to quantize the input.
 
@@ -109,8 +78,6 @@ class InputQuantizer(OnnxLayer):
         self._input = [input_tp]
 
         # Declare weights
-        self._add_weight("input_scale")
-        self._add_weight("offset")
         self._add_weight("range_min", -1.0)
         self._add_weight("range_max", 1.0)
 
@@ -121,10 +88,6 @@ class InputQuantizer(OnnxLayer):
         # Add/initialize weights
         zp_dtype = "int8" if self.input_signed else "uint8"
         self._add_weight("zero_point", value=np.zeros(input_ts.shape[1]), dtype=zp_dtype)
-        if self.weights["input_scale"].size == 0:
-            self.set_weight("input_scale", np.ones((), dtype="float32"))
-        if self.weights["offset"].size == 0:
-            self.set_weight("offset", np.zeros((), dtype="float32"))
 
         # Update perm attribute
         input_ndim = len(input_ts.shape)
@@ -138,45 +101,29 @@ class InputQuantizer(OnnxLayer):
         output_shape = tuple(input_ts.shape[i] for i in perm)
         output_ts = TENSOR_SHAPE(output_shape, np.dtype(zp_dtype))
 
-        # Assert wrong weights format
-        first_channel_expected_shape = (1,) + output_shape[1:2] + (1,) * (input_ndim - 2)
-        if (self.weights["input_scale"].size != 1 and
-                self.weights["input_scale"].shape != first_channel_expected_shape):
-            raise ValueError(f"Unsupported 'input_scale' in {self.name} ({self.base_name}): "
-                             "it must be brodcastable in the channels dimension.")
-        if (self.weights["offset"].size != 1 and
-                self.weights["offset"].shape != first_channel_expected_shape):
-            raise ValueError(f"Unsupported 'offset' in {self.name} ({self.base_name}): "
-                             "it must be brodcastable in the channels dimension.")
         return output_ts
 
     def __quantize__(self, force_fp=False):
-        # Calibration was done by axis=1. Therefore we can squeeze dimension in mean/offset
-        rescale_scale = np.squeeze(self.weights["input_scale"])
-        rescale_offset = np.squeeze(self.weights["offset"])
-
         # Computing quantization parameters
         out_tensor_range = TensorData(lowest=self.weights["range_min"],
                                       highest=self.weights["range_max"])
-        input_scale_zp, output_scale_zp = _compute_quantization_parameters(rescale_scale,
-                                                                           rescale_offset,
-                                                                           out_tensor_range,
-                                                                           self.input_signed)
+
+        # Compute output scale
+        input_scale, input_zp = input_zp_scale(out_tensor_range, allow_zp=not self.input_signed)
 
         # Scale to set in weights is the reciprocal of ONNX calibrated one.
-        input_scale = np.array(1 / input_scale_zp[0], dtype=np.float32)
-
-        # Compute weights to serialize
-        weights = {f"{self.name}_scale": input_scale, f"{self.name}_zp": input_scale_zp[1]}
+        scale = np.array(1 / input_scale, dtype=np.float32)
 
         # Save zero point (used by next layer)
-        self.set_weight("zero_point", output_scale_zp[1])
+        self.set_weight("zero_point", input_zp)
+
+        # Compute weights to serialize
+        weights = {f"{self.name}_scale": scale, f"{self.name}_zp": input_zp}
 
-        output_scale = output_scale_zp[0]
         if force_fp:
-            output_scale = round_to_nearest_pow2(output_scale)
+            input_scale = round_to_nearest_pow2(input_scale)
 
-        return weights, output_scale
+        return weights, input_scale
 
     @staticmethod
     def build_subgraph(op_type):
