@@ -23,16 +23,20 @@ from onnxruntime.quantization import MinMaxCalibrater
 from onnxruntime.quantization.quant_utils import find_by_name
 from onnxruntime_extensions import get_library_path
 
-from .register_patterns import PATTERNS_MAP, CUSTOM_PATTERNS_MAP
+from .register_patterns import PATTERNS_MAP, CUSTOM_PATTERNS_MAP, INPUT_QUANTIZER_PATTERNS
 from .data_reader import CalibrationDataReader
 from .model import ONNXModel
 
 
-def _get_op_types_to_calibrate():
+def _get_op_types_to_calibrate(model):
     # This function computes the set of operation types whose outputs need to be calibrated.
     # These operation types are the last operations in each pattern from
-    # PATTERNS_MAP and CUSTOM_PATTERNS_MAP.
-    return {pattern.pattern[-1] for pattern in PATTERNS_MAP + CUSTOM_PATTERNS_MAP}
+    # PATTERNS_MAP, CUSTOM_PATTERNS_MAP, INPUT_QUANTIZER_PATTERNS and the model
+    # input operation type.
+    # First op_type needs to be added because of the case (X -> BRN_PAD -> Conv) where I need
+    # to calibrate X.
+    return {pattern.pattern[-1] for pattern in PATTERNS_MAP +
+            CUSTOM_PATTERNS_MAP + INPUT_QUANTIZER_PATTERNS}.add(model.graph.node[0].op_type)
 
 
 def _remove_axes_in_reduce_ops(model):
@@ -124,7 +128,7 @@ def calibrate(model,
         # Declare custom MinMax calibrator from model path
         calibrator = CustomMinMaxCalibrator(
             model_path,
-            op_types_to_calibrate=_get_op_types_to_calibrate(),
+            op_types_to_calibrate=_get_op_types_to_calibrate(model),
             augmented_model_path=tmp_dir.joinpath("augmented_model.onnx"),
             use_external_data_format=False,
             symmetric=symmetric,
