@@ -26,9 +26,11 @@ from .. import layers as onnx_qlayers
 from ..graph_tools import generate_node_names, infer_partial_io
 from ..layers.base_layer import (_GLOBAL_REQUIRES_DOWNSCALE, _GLOBAL_REQUIRES_FP_INPUTS,
                                  BRN_OPSET, ONNX_OPSET, IR_VERSION)
+from ..layers.quantizers import get_input_quantizer
+
 from .calibration import calibrate
 from .model import ONNXModel
-from .register_patterns import CUSTOM_PATTERNS_MAP, PATTERNS_MAP
+from .register_patterns import CUSTOM_PATTERNS_MAP, PATTERNS_MAP, INPUT_QUANTIZER_PATTERNS
 from .shape import set_model_shape, check_single_input_model
 from .transforms import sanitize
 
@@ -49,7 +51,7 @@ def is_quantized_node(node):
             ("Quantized" in node.op_type or node.op_type in ("InputQuantizer", "Dequantizer")))
 
 
-def search_block_from_node(node, model, quantize_until=None):
+def search_block_from_node(node, model, quantize_until=None, block_pattern=None):
     """Try to search a quantizable node sequence from a target node.
 
     Args:
@@ -57,17 +59,23 @@ def search_block_from_node(node, model, quantize_until=None):
         model (ONNXModel): the model containing the node.
         quantize_until (str, optional): if provided, limit the search until a node
             whose output matches to it. Defaults to None.
+        block_pattern (list, optional): list of quantizer patterns to use for searching.
+            Defaults to None.
 
     Returns:
         list: a quantizable sequence of nodes. ``None`` if none found.
     """
-    for qpattern in CUSTOM_PATTERNS_MAP + PATTERNS_MAP:
+    if block_pattern is None:
+        block_pattern = CUSTOM_PATTERNS_MAP + PATTERNS_MAP
+
+    graph_outputs = {output.name for output in model.output}
+    for qpattern in block_pattern:
         pattern = qpattern.pattern
         # Try to recognize a sequence of nodes equal to the pattern size
         block_nodes = [node]
         for _ in range(len(pattern) - 1):
-            # End search if quantize_until is provided
-            if block_nodes[-1].output[0] == quantize_until:
+            # End search if quantize_until is provided or if the last node is a graph output.
+            if block_nodes[-1].output[0] in {quantize_until, *graph_outputs}:
                 break
             outbound_nodes = model.get_children(block_nodes[-1])
             # A valid sequence cannot contain multiple outbounds.
@@ -80,7 +88,7 @@ def search_block_from_node(node, model, quantize_until=None):
             # The node search ends because a pattern matchs with block_id.
             # Return all quantized functions that match with the same id.
             target_fn_list = []
-            for qpattern in CUSTOM_PATTERNS_MAP + PATTERNS_MAP:
+            for qpattern in block_pattern:
                 if qpattern.pattern == block_id:
                     target_fn_list.extend(qpattern.f)
             return block_nodes, target_fn_list
@@ -110,6 +118,31 @@ def _set_qlayers_metadata(qmodel, is_fully_quantized):
         _force_fp(quantizer)
 
 
+def _insert_input_quantizer(qmodel, model, graph, tensors_range):
+    input_quantizer_nodes = []
+    graph_input_name = graph.input[0].name
+
+    block_qpattern = search_block_from_node(
+        model.nodes()[0], model, block_pattern=INPUT_QUANTIZER_PATTERNS)
+
+    if block_qpattern:
+        input_quantizer_nodes, _ = block_qpattern
+        # If there are nodes for the input quantizer, update graph_input_name
+        # to the last node output
+        graph_input_name = input_quantizer_nodes[-1].output[0]
+
+    input_quantizer = get_input_quantizer(input_quantizer_nodes, graph, tensors_range)
+    qmodel.quantizers = [Quantizer(input_quantizer, [], graph_input_name)]
+
+    # Remove input quantizer nodes from the model nodes
+    # Note: list(nodes) clone each NodeProto in graph.node field
+    remaining_nodes = list(model.nodes())
+    for node in input_quantizer_nodes:
+        remaining_nodes.remove(node)
+
+    return qmodel, graph_input_name, remaining_nodes
+
+
 def quantize_calibrated(model, tensors_range, quantize_until=None):
     """
     Given a calibrated onnx model and associated tensor ranges, create a quantized onnx
@@ -147,17 +180,15 @@ def quantize_calibrated(model, tensors_range, quantize_until=None):
     qmodel.input.extend(model.input)
     qmodel.set_opset_import(onnx_qlayers.BRN_OPSET.domain, onnx_qlayers.BRN_OPSET.version)
 
-    # Split nodes in blocks.
-    # Note: list(nodes) clone each NodeProto in graph.node field
-    remaining_nodes = list(model.nodes())
+    # Insert InputQuantizer
+    qmodel, graph_input_name, remaining_nodes = _insert_input_quantizer(
+        qmodel, model, graph, tensors_range)
+    output_names = [qmodel.quantizers[0].out_name]
 
     # Start with all nodes linked to the input
     node_queue = []
-    output_names = []
-    qmodel.quantizers = []
-
     for node in model.nodes():
-        if model.get_node_inputs(node) == [graph.input[0].name]:
+        if model.get_node_inputs(node) == [graph_input_name]:
             node_queue.append(node)
     while len(node_queue) > 0:
         # Process the first node in the queue
@@ -194,9 +225,6 @@ def quantize_calibrated(model, tensors_range, quantize_until=None):
             try:
                 qinputs = []
                 for x in model.get_node_inputs(target_node):
-                    if x == model.input[0].name:
-                        # Skip nodes that are linked to inputs (e.g. InputQuantizer).
-                        continue
                     quantizer_id = output_names.index(x)
                     qinputs.append(qmodel.quantizers[quantizer_id].qlayer)
             except ValueError:
@@ -226,20 +254,11 @@ def quantize_calibrated(model, tensors_range, quantize_until=None):
     if len(qmodel.quantizers) <= 1:
         raise RuntimeError("No quantizable pattern found.")
 
-    # Compute each remaining input (those whose nodes are disconnected remaining model)
-    partial_float_in, _ = infer_partial_io(remaining_nodes,
-                                           exclude=list(model.get_initializer_name_set()))
-
-    is_fully_quantized = len(remaining_nodes) == 0
-    # Output needs to be dequantized when there are no remaining_nodes
-    if is_fully_quantized:
-        for node in graph.output:
-            partial_float_in.append(node.name)
-
     # Main loop: quantize qlayers and concatenate them in qnodes
     value_info = qmodel.graph().value_info
 
     # Setting qlayer metadata such as: Downscaling, forcing fixed_point inputs
+    is_fully_quantized = len(remaining_nodes) == 0
     _set_qlayers_metadata(qmodel, is_fully_quantized)
 
     for qidx, quantizer in enumerate(qmodel.quantizers):
@@ -261,6 +280,17 @@ def quantize_calibrated(model, tensors_range, quantize_until=None):
         # Update output name list
         output_names[qidx] = qnode.output[0]
 
+    # Compute each remaining input (those whose nodes are disconnected remaining model)
+    partial_float_in, _ = infer_partial_io(remaining_nodes,
+                                           exclude=list(model.get_initializer_name_set()))
+
+    # Add to remaining inputs those tensors that are an output of the model.
+    # These will be the tensors to be dequantized.
+    graph_outputs = [output.name for output in model.output]
+    for iname in output_names:
+        if iname in graph_outputs and iname not in partial_float_in:
+            partial_float_in.append(iname)
+
     # Add a Dequantizer
     io_deq_map = []
     qlayers = [qmodel.quantizers[output_names.index(iname)].qlayer for iname in partial_float_in]
@@ -274,6 +304,12 @@ def quantize_calibrated(model, tensors_range, quantize_until=None):
     io_deq_map = [(dequantizer.output[i].name, iname) for i, iname in enumerate(partial_float_in)]
     # Register functions in the quantized graph
     qmodel.functions.extend(onnx_qlayers.AKIDA_ONNX_LAYERS)
+    # Compute the expected model outputs.
+    for deq_out, model_out in io_deq_map:
+        try:
+            graph_outputs[graph_outputs.index(model_out)] = deq_out
+        except ValueError:
+            pass
 
     # Finally build the quantized model
     if len(remaining_nodes) > 0:
@@ -284,21 +320,36 @@ def quantize_calibrated(model, tensors_range, quantize_until=None):
                           "Continuing execution.")
         # Extract remaining submodel
         extractor = onnx.utils.Extractor(model.model)
-        remaining_model = extractor.extract_model(partial_float_in, [model.output[0].name])
+        remaining_model = extractor.extract_model(partial_float_in, [x.name for x in model.output])
         # merge_models throws an error if it finds value_info name overlap in both models.
-        # To avoid it, we give priority to those present in the quantized model.
-        for value_info in list(remaining_model.graph.value_info):
-            if qmodel.find_value_info_by_name(value_info.name):
-                remaining_model.graph.value_info.remove(value_info)
+        # To avoid it, we temporarily store value_info and reinsert them after merging.
+        # Note we priorize qmodel value_info in case of name overlap.
+        value_infos = {vi.name: vi for vi in remaining_model.graph.value_info}
+        for vi in list(qmodel.graph().value_info) + list(qmodel.output):
+            value_infos[vi.name] = vi
+        qmodel.graph().ClearField('value_info')
+        remaining_model.graph.ClearField('value_info')
         # Use onnx.compose helper tool to merge the models manually,
         # avoiding some issues (e.g. topological ordering) and removing duplicate functions.
         for pfunc in list(qmodel.functions):
             while (f := find_by_name(pfunc.name, remaining_model.functions)):
                 remaining_model.functions.remove(f)
-
-        qmodel.graph().value_info.extend(qmodel.output)
-        qmodel = onnx.compose.merge_models(qmodel.model, remaining_model, io_map=io_deq_map)
+        # Merge models and sort outputs to match the original model outputs.
+        qmodel = onnx.compose.merge_models(qmodel.model,
+                                           remaining_model,
+                                           io_map=io_deq_map,
+                                           outputs=graph_outputs)
+        # Reinsert value infos.
+        exclude_tensors = {vi.name for vi in [*qmodel.graph.initializer, *qmodel.graph.output]}
+        for node in qmodel.graph.node:
+            for out in node.output:
+                if out not in exclude_tensors and out in value_infos:
+                    qmodel.graph.value_info.append(value_infos[out])
         qmodel = ONNXModel(qmodel)
+    # Sort outputs to match original model ones.
+    graph_outputs = sorted(qmodel.output, key=lambda x: graph_outputs.index(x.name))
+    qmodel.graph().ClearField('output')
+    qmodel.output.extend(graph_outputs)
     return qmodel
 
 
