@@ -27,6 +27,12 @@ QuantizePattern = namedtuple('QuantizerPattern', ['pattern', 'f'])
 CUSTOM_PATTERNS_MAP = []
 _CONV_DW_CONV_FNS = [onnx_qlayers.get_qdepthwise, onnx_qlayers.get_qconv]
 PATTERNS_MAP = [
+    QuantizePattern(("BRNPad", "Conv", "MaxPool", "Relu"), [onnx_qlayers.get_qconv]),
+    QuantizePattern(("BRNPad", "Conv", "MaxPool", "Clip"), [onnx_qlayers.get_qconv]),
+    QuantizePattern(("BRNPad", "Conv", "activation"), [onnx_qlayers.get_qconv]),
+    QuantizePattern(("BRNPad", "Conv", "Relu"), [onnx_qlayers.get_qconv]),
+    QuantizePattern(("BRNPad", "Conv", "Clip"), [onnx_qlayers.get_qconv]),
+    QuantizePattern(("BRNPad", "Conv"), [onnx_qlayers.get_qconv]),
     QuantizePattern(("Conv", "Relu", "GlobalAveragePool"), _CONV_DW_CONV_FNS),
     QuantizePattern(("Conv", "MaxPool", "Relu"), _CONV_DW_CONV_FNS),
     QuantizePattern(("Conv", "GlobalAveragePool"), _CONV_DW_CONV_FNS),
@@ -49,16 +55,18 @@ PATTERNS_MAP = [
     QuantizePattern(("ConvTranspose", "Relu"), [onnx_qlayers.get_qconv_transpose]),
     QuantizePattern(("ConvTranspose", "activation"), [onnx_qlayers.get_qconv_transpose]),
     QuantizePattern(("ConvTranspose",), [onnx_qlayers.get_qconv_transpose]),
-    QuantizePattern(("Cast", "Transpose", "Mul", "Add"), [onnx_qlayers.get_input_quantizer]),
-    QuantizePattern(("Cast", "Transpose", "Mul"), [onnx_qlayers.get_input_quantizer]),
-    QuantizePattern(("Cast", "Mul", "Add"), [onnx_qlayers.get_input_quantizer]),
-    QuantizePattern(("Cast", "Mul",), [onnx_qlayers.get_input_quantizer]),
     QuantizePattern(("BufferTempConv", "Relu"), [onnx_qlayers.get_qbtc]),
     QuantizePattern(("DepthwiseBufferTempConv", "Relu"), [onnx_qlayers.get_qdbtc]),
     QuantizePattern(("BufferTempConv",), [onnx_qlayers.get_qbtc]),
     QuantizePattern(("DepthwiseBufferTempConv",), [onnx_qlayers.get_qdbtc]),
 ]
 
+INPUT_QUANTIZER_PATTERNS = [
+    QuantizePattern(("Cast", "Transpose"), [onnx_qlayers.get_input_quantizer]),
+    QuantizePattern(("Cast",), [onnx_qlayers.get_input_quantizer]),
+    QuantizePattern(("Transpose",), [onnx_qlayers.get_input_quantizer]),
+]
+
 
 @contextmanager
 def custom_pattern_scope(new_patterns):
