@@ -7,7 +7,7 @@ from .convert_matmul_to_gemm import *
 from .convert_min_max_to_clip import *
 from .untranspose_gemm_weights import *
 from .reduce_math_constant_ops import *
-from .insert_rescaling import *
+from .reorder_ops import *
 from .align_rescaling import *
 from .split_concat_and_sum_nodes import *
 from .convert_resize_to_depthwise_transpose import *
@@ -21,3 +21,4 @@ from .replace_conv3d import *
 from .clone_shared_inputs import *
 from .invert_batchnorm_pooling import *
 from .split_dw_nodes import *
+from .convert_reshape_to_transpose import *
