@@ -60,6 +60,13 @@ def convert_min_max_to_clip(model):
             node_value_tp.CopyFrom(onnx.numpy_helper.from_array(
                 np.array(node_value.item(), np.float32), node.input[1]))
 
+            # Remove the value info to infer new shape later as the Clip
+            # requires scalars for min and max and previous value could have
+            # size of 1 but not be scalar.
+            node_value_info = model.find_value_info_by_name(node.input[1])
+            if node_value_info is not None:
+                model.graph().value_info.remove(node_value_info)
+
             if node.op_type == "Min":
                 # Add an empty min. Min should be before Max that's why we swap min and max
                 node.input.append("")
