@@ -34,7 +34,7 @@ from .convert_matmul_to_gemm import convert_matmul_to_gemm
 from .convert_min_max_to_clip import convert_min_max_to_clip
 from .convert_resize_to_depthwise_transpose import convert_resize_to_depthwise_transpose
 from .reduce_math_constant_ops import reduce_math_constant_ops
-from .insert_rescaling import insert_rescaling
+from .reorder_ops import reorder_ops
 from .align_rescaling import align_rescaling
 from .remove_reshape import remove_reshape
 from .squeeze_reshape_to_flatten import convert_squeeze_reshape_to_flatten
@@ -49,6 +49,7 @@ from .invert_batchnorm_pooling import invert_batchnorm_pooling
 from .replace_conv3d import replace_conv3d
 from .clone_shared_inputs import clone_shared_inputs
 from .split_dw_nodes import split_dw_nodes
+from .convert_reshape_to_transpose import convert_reshape_to_transpose
 from ..model import ONNXModel
 from ...graph_tools import get_field
 from ...layers import BRN_OPSET
@@ -85,6 +86,9 @@ def sanitize_base(model):
     # Duplicate initializers that are used more than once
     clone_shared_inputs(model)
 
+    # Convert Reshape to Transpose when possible
+    convert_reshape_to_transpose(model)
+
     # Swaps Pad and Transpose nodes when possible
     swap_pad_transpose(model)
 
@@ -131,13 +135,12 @@ def sanitize_base(model):
     # Fuse batchnorms
     model.rewrite(fuse_batchnorm_rule_set())
 
+    # Reorder model entry operations
+    reorder_ops(model)
+
     # Align rescaling nodes to Conv/Gemm nodes
     align_rescaling(model)
 
-    # Adds a Rescaling node to the input of the model
-    # Note it is applied after align_rescaling to avoid fold Mul into next conv
-    insert_rescaling(model)
-
     # Replace activations
     replace_activations(model)
 
@@ -168,7 +171,7 @@ def sanitize_for_hardware(model):
     insert_identity_convs(model)
 
 
-def sanitize(model):
+def sanitize(model, num_interations=2):
     """Sanitize a model preparing it for quantization.
 
     This is a wrapping successive calls to several model transformations
@@ -176,6 +179,8 @@ def sanitize(model):
 
     Args:
         model(ONNXModel): the input model
+        num_iterations(int, optional): number of times the sanitization loop is repeated.
+            Defaults to 2.
 
     Returns:
         ONNXModel: the sanitized model
@@ -191,56 +196,57 @@ def sanitize(model):
     # Clean inputs/outputs
     model.clean_graph_io()
 
-    # Perform optimization only if model is not quantized
-    if not any(node.domain == "com.brainchip" for node in model.nodes()):
-        with tempfile.TemporaryDirectory(prefix="pre.quant.") as quant_tmp_dir:
-            # To perfom ONNXRuntime optimization, we would like to use
-            # onnxruntime.quantization.quant_pre_process, to optimize the model (when required)
-            # and infer the intermediate shapes.
-            # However, it always expects to read the model from a path. That is why we
-            # save the input model if it is not a path.
-            tmp_model_path = os.path.join(quant_tmp_dir, "model.onnx")
-            model.save_model_to_file(tmp_model_path)
+    for _ in range(num_interations):
+        # Perform optimization only if model is not quantized
+        if not any(node.domain == "com.brainchip" for node in model.nodes()):
+            with tempfile.TemporaryDirectory(prefix="pre.quant.") as quant_tmp_dir:
+                # To perfom ONNXRuntime optimization, we would like to use
+                # onnxruntime.quantization.quant_pre_process, to optimize the model (when required)
+                # and infer the intermediate shapes.
+                # However, it always expects to read the model from a path. That is why we
+                # save the input model if it is not a path.
+                tmp_model_path = os.path.join(quant_tmp_dir, "model.onnx")
+                model.save_model_to_file(tmp_model_path)
 
-            # We employ onnxruntime preprocessing to implement optimization.
-            # There are some issue in onnxruntime (e.g. Constant Node) that
-            # raise exceptions when computing shapes using symbolic_shape_inference.
-            # That is why we bypass the symbolic shape inference and apply first
-            # an optimization that folds these constants, and then apply by shape inference.
-            try:
-                # When symbolic shape inference fails, it leaves behind unused files in the
-                # current directory. To avoid polluting the working directory, temporarily
-                # switch to the temp folder so these files are stored there instead.
-                old_cwd = os.getcwd()
-                os.chdir(quant_tmp_dir)
+                # We employ onnxruntime preprocessing to implement optimization.
+                # There are some issue in onnxruntime (e.g. Constant Node) that
+                # raise exceptions when computing shapes using symbolic_shape_inference.
+                # That is why we bypass the symbolic shape inference and apply first
+                # an optimization that folds these constants, and then apply by shape inference.
+                try:
+                    # When symbolic shape inference fails, it leaves behind unused files in the
+                    # current directory. To avoid polluting the working directory, temporarily
+                    # switch to the temp folder so these files are stored there instead.
+                    old_cwd = os.getcwd()
+                    os.chdir(quant_tmp_dir)
 
-                quant_pre_process(tmp_model_path, tmp_model_path, skip_symbolic_shape=True)
-                quant_pre_process(tmp_model_path, tmp_model_path)
-            except Exception:
-                # Skip optimization and/or shape inference if it fails
-                warnings.warn("ONNXRuntime optimization failed. This is not a critical error, "
-                              "skipping optimization and/or shape inference and continuing "
-                              "execution.")
-            finally:
-                # restore cwd
-                os.chdir(old_cwd)
+                    quant_pre_process(tmp_model_path, tmp_model_path, skip_symbolic_shape=True)
+                    quant_pre_process(tmp_model_path, tmp_model_path)
+                except Exception:
+                    # Skip optimization and/or shape inference if it fails
+                    warnings.warn("ONNXRuntime optimization failed. This is not a critical error, "
+                                  "skipping optimization and/or shape inference and continuing "
+                                  "execution.")
+                finally:
+                    # restore cwd
+                    os.chdir(old_cwd)
 
-            # Load the model
-            model = onnx.load_model(Path(tmp_model_path))
+                # Load the model
+                model = onnx.load_model(Path(tmp_model_path))
 
-            # If ONNXRuntime optimization fails, shapes will not be inferred.
-            # Consequently we use ONNX shape inference module to TRY to ensure that shapes
-            # will be inferred.
-            model = ONNXModel(onnx.shape_inference.infer_shapes(model, check_type=True))
+                # If ONNXRuntime optimization fails, shapes will not be inferred.
+                # Consequently we use ONNX shape inference module to TRY to ensure that shapes
+                # will be inferred.
+                model = ONNXModel(onnx.shape_inference.infer_shapes(model, check_type=True))
 
-    # Sanitizers expected a float input.
-    _assert_float_inputs(model)
+        # Sanitizers expected a float input.
+        _assert_float_inputs(model)
 
-    # Apply onnxscript optimization
-    _apply_onnxscript_optimization(model)
+        # Apply onnxscript optimization
+        _apply_onnxscript_optimization(model)
 
-    sanitize_base(model)
-    sanitize_for_hardware(model)
+        sanitize_base(model)
+        sanitize_for_hardware(model)
 
     # ONNXRuntime optimization will mess with BRN_OPSET.version setting it to 1000 which will cause
     # issues when manipulating models (merging during quantization). For that reason, the opset is
