@@ -2,6 +2,3 @@ from .qtensor import *
 from .qfloat import *
 from .fixed_point import *
 from .tf_dispatch import *
-
-BASE_BUFFER_BITWIDTH = 24
-MAX_BUFFER_BITWIDTH = 32
