@@ -446,7 +446,6 @@ class FixedPoint(QTensor):
             f"Unsupported operand type(s) for /: 'FixedPoint' and '{type(other)}'")
 
     def __getitem__(self, idx):
-
         r"""Retrieve a slice or element from the FixedPoint tensor.
 
         This function allows for slicing or indexing the FixedPoint tensor in a similar way
@@ -530,22 +529,3 @@ class FixedPoint(QTensor):
                 f"Unsupported operand type(s) for <=: 'FixedPoint' and '{type(other)}'")
         _, s_values, o_values = self._align_values(other)
         return s_values <= o_values
-
-    def abs(self):
-        """Returns the absolute value of the FixedPoint
-
-        Returns:
-            :obj:`FixedPoint`: the absolute value.
-        """
-        return FixedPoint(tf.math.abs(self.values), self.value_bits, self.frac_bits)
-
-    def floor(self):
-        """Floors the FixedPoint
-
-        Returns:
-            tuple(:obj:`FixedPoint`, tf.Tensor): a new FixedPoint without
-            fractional bits and the shift that was applied.
-        """
-        # Divide values to remove fractional bits
-        values = FixedPoint._rshift(self.values, self.frac_bits)
-        return FixedPoint(values, self.value_bits, 0), self.frac_bits
