@@ -39,8 +39,8 @@ class QFloat(QTensor):
         fp (:obj:`FixedPoint`): a FixedPoint of values
         scales (tf.Tensor): a Tensor of scales
     """
-    fp: FixedPoint = FixedPoint(1.0, 32, 0)
-    scales: tf.Tensor = 1.0
+    fp: FixedPoint
+    scales: tf.Tensor
 
     def __init__(self, fp, scales):
         self.scales = tf.cast(scales, tf.float32)
