@@ -15,7 +15,7 @@
 # limitations under the License.
 # ******************************************************************************
 import tensorflow as tf
-import keras
+import tf_keras as keras
 
 from ...debugging import assert_rank_at_least
 from ..fixed_point import FixedPoint
@@ -32,6 +32,8 @@ def fp_fp_matmul(a: FixedPoint,
                  a_is_sparse=False,
                  b_is_sparse=False,
                  output_type=None,
+                 grad_a=False,
+                 grad_b=False,
                  name=None):
     """Multiplies matrix `a` by matrix `b`, producing `a` * `b`.
 
@@ -52,6 +54,8 @@ def fp_fp_matmul(a: FixedPoint,
             original tf.matmul. Defaults to False.
         output_type (NoneType, optional): must be None, argument kept for compatibility
             with original tf.matmul. Defaults to None.
+        grad_a (bool, optional): Set it to `True` to hint that Tensor `a` is for the backward pass.
+        grad_b (bool, optional): Set it to `True` to hint that Tensor `b` is for the backward pass.
         name (str, optional): the name for the operation. Defaults to None.
 
     Returns:
@@ -66,6 +70,10 @@ def fp_fp_matmul(a: FixedPoint,
     if output_type is not None:
         raise ValueError(
             f"Unsupported argument: output_type, provided {output_type}")
+    if grad_a:
+        raise ValueError(f"Unsupported argument: grad_a, provided {grad_a}")
+    if grad_b:
+        raise ValueError(f"Unsupported argument: grad_a, provided {grad_b}")
 
     # We don't support matmul between vectors
     assert_rank_at_least(a.values, 2)
@@ -94,6 +102,8 @@ def fp_qf_matmul(a: FixedPoint,
                  a_is_sparse=False,
                  b_is_sparse=False,
                  output_type=None,
+                 grad_a=False,
+                 grad_b=False,
                  name=None):
     """Multiplies matrix `a` by matrix `b`, producing `a` * `b`.
 
@@ -114,6 +124,8 @@ def fp_qf_matmul(a: FixedPoint,
             original tf.matmul. Defaults to False.
         output_type (NoneType, optional): must be None, argument kept for compatibility
             with original tf.matmul. Defaults to None.
+        grad_a (bool, optional): Set it to `True` to hint that Tensor `a` is for the backward pass.
+        grad_b (bool, optional): Set it to `True` to hint that Tensor `b` is for the backward pass.
         name (str, optional): the name for the operation. Defaults to None.
 
     Returns:
@@ -124,7 +136,7 @@ def fp_qf_matmul(a: FixedPoint,
         b.assert_per_tensor()
 
     # Do matmul on a and b inner FixedPoint
-    fp_prod = tf.matmul(a, b.fp, transpose_a, transpose_b, adjoint_a, adjoint_b, name)
+    fp_prod = tf.matmul(a, b.fp, transpose_a, transpose_b, adjoint_a, adjoint_b, name=name)
 
     # Return a new QFloat with the same scales
     return QFloat(fp_prod, b.scales)
@@ -140,6 +152,8 @@ def qf_qf_matmul(a: QFloat,
                  a_is_sparse=False,
                  b_is_sparse=False,
                  output_type=None,
+                 grad_a=False,
+                 grad_b=False,
                  name=None):
     """Multiplies matrix `a` by matrix `b`, producing `a` * `b`.
 
@@ -160,6 +174,8 @@ def qf_qf_matmul(a: QFloat,
             original tf.matmul. Defaults to False.
         output_type (NoneType, optional): must be None, argument kept for compatibility
             with original tf.matmul. Defaults to None.
+        grad_a (bool, optional): Set it to `True` to hint that Tensor `a` is for the backward pass.
+        grad_b (bool, optional): Set it to `True` to hint that Tensor `b` is for the backward pass.
         name (str, optional): the name for the operation. Defaults to None.
 
     Returns:
@@ -172,7 +188,7 @@ def qf_qf_matmul(a: QFloat,
         b.assert_per_tensor()
 
     # Do matmul on a and b inner FixedPoint
-    fp_prod = tf.matmul(a.fp, b.fp, transpose_a, transpose_b, adjoint_a, adjoint_b, name)
+    fp_prod = tf.matmul(a.fp, b.fp, transpose_a, transpose_b, adjoint_a, adjoint_b, name=name)
 
     # Return a new QFloat with multiplied scales
     return QFloat(fp_prod, a.scales * b.scales)
