@@ -510,7 +510,7 @@ def qf_tile(input: QFloat, multiples, name=None):
 
 
 @tf.experimental.dispatch_for_api(tf.zeros_like)
-def fp_zeros_like(input: FixedPoint, dtype=None, name=None):
+def fp_zeros_like(input: FixedPoint, dtype=None, name=None, layout=None):
     """Creates a FixedPoint with all elements set to zero and the same frac_bits and shape as
     input.
 
@@ -520,19 +520,22 @@ def fp_zeros_like(input: FixedPoint, dtype=None, name=None):
             `float64`, `int8`, `uint8`, `int16`, `uint16`, `int32`, `int64`, `complex64`,
             `complex128`, `bool` or `string` (optional).
         name (str, optional): name of the output `Tensor`. Defaults to None.
+        layout (`tf.experimental.dtensor.Layout`, optional): If provided, the result is a
+            [DTensor](https://www.tensorflow.org/guide/dtensor_overview) with the provided layout.
+            Defaults to None.
 
     Returns:
         FixedPoint: a tensor with the same data as `input`.
     """
     # Create a tf.Tensor with all element set to zeros
-    values = tf.zeros_like(input.values, dtype, name)
+    values = tf.zeros_like(input.values, dtype, name, layout)
 
     # Return a new FixedPoint
     return FixedPoint(values, input.value_bits, input.frac_bits)
 
 
 @tf.experimental.dispatch_for_api(tf.zeros_like)
-def qf_zeros_like(input: QFloat, dtype=None, name=None):
+def qf_zeros_like(input: QFloat, dtype=None, name=None, layout=None):
     """Creates a QFloat with all elements set to zero and the same scales and shape as
     input.
 
@@ -542,13 +545,15 @@ def qf_zeros_like(input: QFloat, dtype=None, name=None):
             `float64`, `int8`, `uint8`, `int16`, `uint16`, `int32`, `int64`, `complex64`,
             `complex128`, `bool` or `string` (optional).
         name (str, optional): name of the output `Tensor`. Defaults to None.
-
+        layout (`tf.experimental.dtensor.Layout`, optional): If provided, the result is a
+            [DTensor](https://www.tensorflow.org/guide/dtensor_overview) with the provided layout.
+            Defaults to None.
 
     Returns:
         QFloat: a tensor with the same data as `input`.
     """
     # Create a tf.Tensor with all element set to zeros
-    values = tf.zeros_like(input.fp, dtype, name)
+    values = tf.zeros_like(input.fp, dtype, name, layout)
 
     # Return a new QFloat
     return QFloat(values, input.scales)
